@@ -1,5 +1,5 @@
 /* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -23,19 +23,15 @@
 #include "loudness.h"
 #include "mtrand.h"
 #include "nortsong.h"
-#include "nortvars.h"
 #include "opentyr.h"
+#include "player.h"
 #include "varz.h"
 #include "vga256d.h"
 #include "video.h"
 #include "video_scale.h"
 
-#include <stdbool.h>
-#include <stdio.h>
-#include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 
 
 /* Configuration Load/Save handler */
@@ -47,7 +43,7 @@ const JE_byte cryptKey[10] = /* [1..10] */
 
 const JE_KeySettingType defaultKeySettings =
 {
-	SDLK_UP, SDLK_DOWN, SDLK_LEFT, SDLK_RIGHT, SDLK_SPACE, SDLK_KP_ENTER, SDLK_LCTRL, SDLK_LALT
+	SDLK_UP, SDLK_DOWN, SDLK_LEFT, SDLK_RIGHT, SDLK_SPACE, SDLK_RETURN, SDLK_LCTRL, SDLK_LALT
 /*	72, 80, 75, 77, 57, 28, 29, 56*/
 };
 
@@ -142,40 +138,25 @@ const JE_EditorItemAvailType initialItemAvail =
 JE_boolean smoothies[9] = /* [1..9] */
 { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 
-
 JE_byte starShowVGASpecialCode;
 
-/* Stars */
-StarDatType starDat[MAX_STARS]; /* [1..Maxstars] */
-JE_word starY;
-
-
 /* CubeData */
 JE_word lastCubeMax, cubeMax;
 JE_word cubeList[4]; /* [1..4] */
 
 /* High-Score Stuff */
-JE_boolean gameHasRepeated;
+JE_boolean gameHasRepeated;  // can only get highscore on first play-through
 
 /* Difficulty */
-JE_shortint difficultyLevel, oldDifficultyLevel, initialDifficulty;
+JE_shortint difficultyLevel, oldDifficultyLevel,
+            initialDifficulty;  // can only get highscore on initial episode
 
 /* Player Stuff */
-JE_longint score, score2;
+uint    power, lastPower, powerAdd;
+JE_byte shieldWait, shieldT;
 
-JE_integer    power, lastPower, powerAdd;
-JE_PItemsType pItems, pItemsPlayer2, pItemsBack, pItemsBack2;
-JE_shortint   shield, shieldMax, shieldSet;
-JE_shortint   shield2, shieldMax2;
-JE_integer    armorLevel, armorLevel2;
-JE_byte       shieldWait, shieldT;
-
-JE_byte          shotRepeat[11], shotMultiPos[11]; /* [1..11] */  /* 7,8 = Superbomb */
-JE_byte          portConfig[10]; /* [1..10] */
+JE_byte          shotRepeat[11], shotMultiPos[11];
 JE_boolean       portConfigChange, portConfigDone;
-JE_PortPowerType portPower, lastPortPower;
-
-JE_boolean resetVersion;
 
 /* Level Data */
 char    lastLevelName[11], levelName[11]; /* string [10] */
@@ -211,12 +192,10 @@ JE_boolean superPause = false;
 JE_boolean explosionTransparent,
            youAreCheating,
            displayScore,
-           soundHasChanged,
            background2, smoothScroll, wild, superWild, starActive,
            topEnemyOver,
            skyEnemyOverAll,
-           background2notTransparent,
-           tempb;
+           background2notTransparent;
 
 JE_byte soundEffects; // dummy value for config
 JE_byte versionNum;   /* SW 1.0 and SW/Reg 1.1 = 0 or 1
@@ -234,53 +213,131 @@ JE_SaveGameTemp saveTemp;
 
 JE_word editorLevel;   /*Initial value 800*/
 
-const JE_byte StringCryptKey[10] = {99, 204, 129, 63, 255, 71, 19, 25, 62, 1};
 
-void JE_decryptString( char *s, JE_byte len )
+cJSON *load_json( const char *filename )
 {
-	int i;
+	FILE *f = dir_fopen_warn(get_user_directory(), filename, "rb");
+	if (f == NULL)
+		return NULL;
+	
+	size_t buffer_len = ftell_eof(f);
+	char *buffer = malloc(buffer_len + 1);
+	
+	fread(buffer, 1, buffer_len, f);
+	buffer[buffer_len] = '\0';
+	
+	fclose(f);
+	
+	cJSON *root = cJSON_Parse(buffer);
+	
+	free(buffer);
+	
+	return root;
+}
 
-	for (i = len-1; i >= 0; i--)
+void save_json( cJSON *root, const char *filename )
+{
+	FILE *f = dir_fopen_warn(get_user_directory(), filename, "w+");
+	if (f == NULL)
+		return;
+	
+	char *buffer = cJSON_Print(root);
+	
+	if (buffer != NULL)
 	{
-		s[i] ^= StringCryptKey[((i+1) % 10)];
-		if (i > 0)
-		{
-			s[i] ^= s[i-1];
-		}
+		fputs(buffer, f);
+		free(buffer);
 	}
+	
+	fclose(f);
 }
 
-void JE_readCryptLn( FILE* f, char *s )
+bool load_opentyrian_config( void )
 {
-	int size;
-
-	size = getc(f);
-	if (size < 0)
-		size = 0;
-	efread(s, 1, size, f);
-	s[size] = '\0';
-	JE_decryptString(s, size);
-}
-
-void JE_skipCryptLn( FILE* f )
-{
-	int size;
-
-	size = getc(f);
-	fseek(f, size, SEEK_CUR);
-}
-
-
-void JE_setupStars( void )
-{
-	int z;
-
-	for (z = MAX_STARS; z--; )
+	// defaults
+	fullscreen_enabled = false;
+	set_scaler_by_name("Scale2x");
+	
+	cJSON *root = load_json("opentyrian.conf");
+	if (root == NULL)
+		return false;
+	
+	cJSON *section = cJSON_GetObjectItem(root, "video");
+	if (section != NULL)
 	{
-		starDat[z].sLoc = (mt_rand() % 320) + (mt_rand() % 200) * VGAScreen->pitch;
-		starDat[z].sMov = ((mt_rand() % 3) + 2) * VGAScreen->pitch;
-		starDat[z].sC = (mt_rand() % 16) + (9 * 16);
+		cJSON *setting;
+		
+		if ((setting = cJSON_GetObjectItem(section, "fullscreen")))
+			fullscreen_enabled = (setting->type == cJSON_True);
+		
+		if ((setting = cJSON_GetObjectItem(section, "scaler")))
+			set_scaler_by_name(setting->valuestring);
 	}
+	
+	cJSON_Delete(root);
+	
+	return true;
+}
+
+bool save_opentyrian_config( void )
+{
+	cJSON *root = load_json("opentyrian.conf");
+	if (root == NULL)
+		root = cJSON_CreateObject();
+	
+	cJSON *section;
+	
+	section = cJSON_CreateOrGetObjectItem(root, "video");
+	cJSON_ForceType(section, cJSON_Object);
+	
+	{
+		cJSON *setting;
+		
+		setting = cJSON_CreateOrGetObjectItem(section, "fullscreen");
+		cJSON_SetBoolean(setting, fullscreen_enabled);
+		
+		setting = cJSON_CreateOrGetObjectItem(section, "scaler");
+		cJSON_SetString(setting, scalers[scaler].name);
+	}
+	
+	save_json(root, "opentyrian.conf");
+	
+	cJSON_Delete(root);
+	
+	return true;
+}
+
+static void playeritems_to_pitems( JE_PItemsType pItems, PlayerItems *items, JE_byte initial_episode_num )
+{
+	pItems[0]  = items->weapon[FRONT_WEAPON].id;
+	pItems[1]  = items->weapon[REAR_WEAPON].id;
+	pItems[2]  = items->super_arcade_mode;
+	pItems[3]  = items->sidekick[LEFT_SIDEKICK];
+	pItems[4]  = items->sidekick[RIGHT_SIDEKICK];
+	pItems[5]  = items->generator;
+	pItems[6]  = items->sidekick_level;
+	pItems[7]  = items->sidekick_series;
+	pItems[8]  = initial_episode_num;
+	pItems[9]  = items->shield;
+	pItems[10] = items->special;
+	pItems[11] = items->ship;
+}
+
+static void pitems_to_playeritems( PlayerItems *items, JE_PItemsType pItems, JE_byte *initial_episode_num )
+{
+	items->weapon[FRONT_WEAPON].id  = pItems[0];
+	items->weapon[REAR_WEAPON].id   = pItems[1];
+	items->super_arcade_mode        = pItems[2];
+	items->sidekick[LEFT_SIDEKICK]  = pItems[3];
+	items->sidekick[RIGHT_SIDEKICK] = pItems[4];
+	items->generator                = pItems[5];
+	items->sidekick_level           = pItems[6];
+	items->sidekick_series          = pItems[7];
+	if (initial_episode_num != NULL)
+		*initial_episode_num        = pItems[8];
+	items->shield                   = pItems[9];
+	items->special                  = pItems[10];
+	items->ship                     = pItems[11];
 }
 
 void JE_saveGame( JE_byte slot, const char *name )
@@ -290,21 +347,22 @@ void JE_saveGame( JE_byte slot, const char *name )
 	saveFiles[slot-1].level = saveLevel;
 	
 	if (superTyrian)
-		pItems[P_SUPERARCADE] = SA_SUPERTYRIAN;
+		player[0].items.super_arcade_mode = SA_SUPERTYRIAN;
 	else if (superArcadeMode == SA_NONE && onePlayerAction)
-		pItems[P_SUPERARCADE] = SA_ARCADE;
+		player[0].items.super_arcade_mode = SA_ARCADE;
 	else
-		pItems[P_SUPERARCADE] = superArcadeMode;
+		player[0].items.super_arcade_mode = superArcadeMode;
 	
-	memcpy(&saveFiles[slot-1].items, &pItems, sizeof(pItems));
+	playeritems_to_pitems(saveFiles[slot-1].items, &player[0].items, initial_episode_num);
 	
 	if (twoPlayerMode)
-		memcpy(&saveFiles[slot-1].lastItems, &pItemsPlayer2, sizeof(pItemsPlayer2));
+		playeritems_to_pitems(saveFiles[slot-1].lastItems, &player[1].items, 0);
 	else
-		memcpy(&saveFiles[slot-1].lastItems, &pItemsBack2, sizeof(pItemsBack2));
+		playeritems_to_pitems(saveFiles[slot-1].lastItems, &player[0].last_items, 0);
+	
+	saveFiles[slot-1].score  = player[0].cash;
+	saveFiles[slot-1].score2 = player[1].cash;
 	
-	saveFiles[slot-1].score  = score;
-	saveFiles[slot-1].score2 = score2;
 	memcpy(&saveFiles[slot-1].levelName, &lastLevelName, sizeof(lastLevelName));
 	saveFiles[slot-1].cubes  = lastCubeMax;
 
@@ -313,7 +371,7 @@ void JE_saveGame( JE_byte slot, const char *name )
 		temp = episodeNum - 1;
 		if (temp < 1)
 		{
-			temp = 4; /* JE: {Episodemax is 4 for completion purposes} */
+			temp = EPISODE_AVAILABLE; /* JE: {Episodemax is 4 for completion purposes} */
 		}
 		saveFiles[slot-1].episode = temp;
 	}
@@ -328,19 +386,18 @@ void JE_saveGame( JE_byte slot, const char *name )
 	saveFiles[slot-1].input2 = inputDevice[1];
 
 	strcpy(saveFiles[slot-1].name, name);
-
-	for (x = 0; x < 2; x++)
+	
+	for (uint port = 0; port < 2; ++port)
 	{
-		saveFiles[slot-1].power[x] = portPower[x];
+		// if two-player, use first player's front and second player's rear weapon
+		saveFiles[slot-1].power[port] = player[twoPlayerMode ? port : 0].items.weapon[port].power;
 	}
-
+	
 	JE_saveConfiguration();
 }
 
 void JE_loadGame( JE_byte slot )
 {
-	JE_byte temp5;
-
 	superTyrian = false;
 	onePlayerAction = false;
 	twoPlayerMode = false;
@@ -351,9 +408,10 @@ void JE_loadGame( JE_byte slot )
 	gameHasRepeated   = saveFiles[slot-1].gameHasRepeated;
 	twoPlayerMode     = (slot-1) > 10;
 	difficultyLevel   = saveFiles[slot-1].difficulty;
-	memcpy(&pItems, &saveFiles[slot-1].items, sizeof(pItems));
 	
-	superArcadeMode = pItems[P_SUPERARCADE];
+	pitems_to_playeritems(&player[0].items, saveFiles[slot-1].items, &initial_episode_num);
+	
+	superArcadeMode = player[0].items.super_arcade_mode;
 	
 	if (superArcadeMode == SA_SUPERTYRIAN)
 		superTyrian = true;
@@ -364,23 +422,25 @@ void JE_loadGame( JE_byte slot )
 	
 	if (twoPlayerMode)
 	{
-		memcpy(&pItemsPlayer2, &saveFiles[slot-1].lastItems, sizeof(pItemsPlayer2));
 		onePlayerAction = false;
+		
+		pitems_to_playeritems(&player[1].items, saveFiles[slot-1].lastItems, NULL);
 	}
 	else
 	{
-		memcpy(&pItemsBack2, &saveFiles[slot-1].lastItems, sizeof(pItemsBack2));
+		pitems_to_playeritems(&player[0].last_items, saveFiles[slot-1].lastItems, NULL);
 	}
 
 	/* Compatibility with old version */
-	if (pItemsPlayer2[P2_SIDEKICK_MODE] < 101)
+	if (player[1].items.sidekick_level < 101)
 	{
-		pItemsPlayer2[P2_SIDEKICK_MODE] = 101;
-		pItemsPlayer2[P2_SIDEKICK_TYPE] = pItemsPlayer2[P_LEFT_SIDEKICK];
+		player[1].items.sidekick_level = 101;
+		player[1].items.sidekick_series = player[1].items.sidekick[LEFT_SIDEKICK];
 	}
-
-	score       = saveFiles[slot-1].score;
-	score2      = saveFiles[slot-1].score2;
+	
+	player[0].cash = saveFiles[slot-1].score;
+	player[1].cash = saveFiles[slot-1].score2;
+	
 	mainLevel   = saveFiles[slot-1].level;
 	cubeMax     = saveFiles[slot-1].cubes;
 	lastCubeMax = cubeMax;
@@ -389,27 +449,28 @@ void JE_loadGame( JE_byte slot )
 	inputDevice[0] = saveFiles[slot-1].input1;
 	inputDevice[1] = saveFiles[slot-1].input2;
 
-	for (temp = 0; temp < 2; temp++)
+	for (uint port = 0; port < 2; ++port)
 	{
-		portPower[temp] = saveFiles[slot-1].power[temp];
+		// if two-player, use first player's front and second player's rear weapon
+		player[twoPlayerMode ? port : 0].items.weapon[port].power = saveFiles[slot-1].power[port];
 	}
-
-	temp5 = saveFiles[slot-1].episode;
+	
+	int episode = saveFiles[slot-1].episode;
 
 	memcpy(&levelName, &saveFiles[slot-1].levelName, sizeof(levelName));
 
 	if (strcmp(levelName, "Completed") == 0)
 	{
-		if (temp5 == 4)
+		if (episode == EPISODE_AVAILABLE)
 		{
-			temp5 = 1;
-		} else if (temp5 < 4) {
-			temp5++;
+			episode = 1;
+		} else if (episode < EPISODE_AVAILABLE) {
+			episode++;
 		}
-		/* Increment 1-3 to 2-4.  Episode 4 goes to 1.  Episode 5 stands still. */
+		/* Increment episode.  Episode EPISODE_AVAILABLE goes to 1. */
 	}
 
-	JE_initEpisode(temp5);
+	JE_initEpisode(episode);
 	saveLevel = mainLevel;
 	memcpy(&lastLevelName, &levelName, sizeof(levelName));
 }
@@ -655,7 +716,7 @@ const char *get_user_directory( void )
 #endif // TARGET_UNIX
 	}
 	
-	return "/home/user/.opentyrian";
+	return user_dir;
 }
 #endif // TARGET_MACOSX
 
@@ -671,7 +732,7 @@ void JE_loadConfiguration( void )
 	int y;
 	
 	fi = dir_fopen_warn(get_user_directory(), "tyrian.cfg", "rb");
-	if (fi && ftell_eof(fi) == 20 + sizeof(keySettings) + 2)
+	if (fi && ftell_eof(fi) == 20 + sizeof(keySettings))
 	{
 		/* SYN: I've hardcoded the sizes here because the .CFG file format is fixed
 		   anyways, so it's not like they'll change. */
@@ -681,13 +742,9 @@ void JE_loadConfiguration( void )
 		
 		efread(&inputDevice_, 1, 1, fi);
 		efread(&jConfigure, 1, 1, fi);
-
+		
 		efread(&versionNum, 1, 1, fi);
-		if (resetVersion)
-		{
-			versionNum = 2; /* JE: {Shareware 1.0 and Registered 1.1 = 1} */
-		}
-
+		
 		efread(&processorType, 1, 1, fi);
 		efread(&midiPort, 1, 1, fi);
 		efread(&soundEffects, 1, 1, fi);
@@ -695,23 +752,13 @@ void JE_loadConfiguration( void )
 		efread(&difficultyLevel, 1, 1, fi);
 		
 		efread(joyButtonAssign, 1, 4, fi);
-
+		
 		efread(&tyrMusicVolume, 2, 1, fi);
 		efread(&fxVolume, 2, 1, fi);
-
-		efread(&inputDevice[0], 1, 1, fi);
-		efread(&inputDevice[1], 1, 1, fi);
-
+		
+		efread(inputDevice, 1, 2, fi);
+		
 		efread(keySettings, sizeof(*keySettings), COUNTOF(keySettings), fi);
-
-		/* display settings */
-		Uint8 temp;
-		
-		efread(&temp, 1, 1, fi);
-		fullscreen_enabled = (temp == true);
-		
-		efread(&temp, 1, 1, fi);
-		scaler = temp;
 		
 		fclose(fi);
 	}
@@ -726,10 +773,10 @@ void JE_loadConfiguration( void )
 		gammaCorrection = 0;
 		processorType = 3;
 		gameSpeed = 4;
-		
-		fullscreen_enabled = false;
 	}
 	
+	load_opentyrian_config();
+	
 	if (tyrMusicVolume > 255)
 		tyrMusicVolume = 255;
 	if (fxVolume > 255)
@@ -808,7 +855,7 @@ void JE_loadConfiguration( void )
 
 		/* SYN: This is truncating to bytes. I have no idea what this is doing or why. */
 		/* TODO: Figure out what this is about and make sure it isn't broked. */
-		editorLevel = JE_btow((JE_byte) (SIZEOF_SAVEGAMETEMP - 6), (JE_byte) (SIZEOF_SAVEGAMETEMP - 5));
+		editorLevel = (saveTemp[SIZEOF_SAVEGAMETEMP - 5] << 8) | saveTemp[SIZEOF_SAVEGAMETEMP - 6];
 
 		fclose(fi);
 	} else {
@@ -951,29 +998,21 @@ void JE_saveConfiguration( void )
 		efwrite(&gammaCorrection, 1, 1, f);
 		efwrite(&difficultyLevel, 1, 1, f);
 		efwrite(joyButtonAssign, 1, 4, f);
-
-		efwrite(&tyrMusicVolume, 1, 2, f);
-		efwrite(&fxVolume, 1, 2, f);
-
-		efwrite(&inputDevice[0], 1, 1, f);
-		efwrite(&inputDevice[1], 1, 1, f);
-
+		
+		efwrite(&tyrMusicVolume, 2, 1, f);
+		efwrite(&fxVolume, 2, 1, f);
+		
+		efwrite(inputDevice, 1, 2, f);
+		
 		efwrite(keySettings, sizeof(*keySettings), COUNTOF(keySettings), f);
-
-		/* display settings */
-		Uint8 temp;
-		
-		temp = fullscreen_enabled;
-		efwrite(&temp, 1, 1, f);
-		
-		temp = scaler;
-		efwrite(&temp, 1, 1, f);
 		
 		fclose(f);
-#if (_BSD_SOURCE || _XOPEN_SOURCE >= 500)
-		sync();
-#endif
 	}
+	
+	save_opentyrian_config();
+	
+#if (_BSD_SOURCE || _XOPEN_SOURCE >= 500)
+	sync();
+#endif
 }
 
-// kate: tab-width 4; vim: set noet:
