@@ -1,5 +1,5 @@
 /* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -17,19 +17,19 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #include "file.h"
-#include "fm_synth.h"
 #include "lds_play.h"
 #include "loudness.h"
-#include "nortsong.h"
 #include "opentyr.h"
 
 #include <assert.h>
 
+const unsigned char op_table[9] = {0x00, 0x01, 0x02, 0x08, 0x09, 0x0a, 0x10, 0x11, 0x12};
+
 /* A substantial amount of this code has been copied and adapted from adplug.
    Thanks, guys! Adplug is awesome! :D */
 
 /* Note frequency table (16 notes / octave) */
-const Uint16 frequency[] = {
+static const Uint16 frequency[(13 * 15) - 3] = {
   343, 344, 345, 347, 348, 349, 350, 352, 353, 354, 356, 357, 358,
   359, 361, 362, 363, 365, 366, 367, 369, 370, 371, 373, 374, 375,
   377, 378, 379, 381, 382, 384, 385, 386, 388, 389, 391, 392, 393,
@@ -48,7 +48,7 @@ const Uint16 frequency[] = {
 };
 
 /* Vibrato (sine) table */
-const Uint8 vibtab[] = {
+static const Uint8 vibtab[25 + (13 * 3)] = {
   0, 13, 25, 37, 50, 62, 74, 86, 98, 109, 120, 131, 142, 152, 162,
   171, 180, 189, 197, 205, 212, 219, 225, 231, 236, 240, 244, 247,
   250, 252, 254, 255, 255, 255, 254, 252, 250, 247, 244, 240, 236,
@@ -57,7 +57,7 @@ const Uint8 vibtab[] = {
 };
 
 /* Tremolo (sine * sine) table */
-const Uint8 tremtab[] = {
+static const Uint8 tremtab[128] = {
   0, 0, 1, 1, 2, 4, 5, 7, 10, 12, 15, 18, 21, 25, 29, 33, 37, 42, 47,
   52, 57, 62, 67, 73, 79, 85, 90, 97, 103, 109, 115, 121, 128, 134,
   140, 146, 152, 158, 165, 170, 176, 182, 188, 193, 198, 203, 208,
@@ -69,23 +69,18 @@ const Uint8 tremtab[] = {
   25, 21, 18, 15, 12, 10, 7, 5, 4, 2, 1, 1, 0
 };
 
-const Uint16 frequency[(13 * 15) - 3];
-const Uint8 vibta[25 + (13 * 3)];
-const Uint8 tremtab[128];
-const Uint16 maxsound, maxpos;
+static const Uint16 maxsound = 0x3f, maxpos = 0xff;
 
-SoundBank *soundbank = NULL;
-Channel channel[9];
-Position *positions = NULL;
+static SoundBank *soundbank = NULL;
+static Channel channel[9];
+static Position *positions = NULL;
+
+static Uint8 fmchip[0xff], jumping, fadeonoff, allvolume, hardfade, tempo_now, pattplay, tempo, regbd, chandelay[9], mode, pattlen;
+static Uint16 posplay, jumppos, speed;
+static Uint16 *patterns = NULL;
+static Uint16 numpatch, numposi, mainvolume;
 
-Uint8 fmchip[0xff], jumping, fadeonoff, allvolume, hardfade, tempo_now, pattplay, tempo, regbd, chandelay[9], mode, pattlen;
-Uint16 posplay, jumppos, speed;
-Uint16 *patterns = NULL;
 bool playing, songlooped;
-Uint16 numpatch, numposi, patterns_size, mainvolume;
-
-const Uint16 maxsound = 0x3f, maxpos = 0xff;
-Uint8 *read_pos;
 
 bool lds_load( FILE *f, unsigned int music_offset, unsigned int music_size )
 {
@@ -213,7 +208,7 @@ void lds_rewind( void )
 	memset(fmchip, 0, sizeof(fmchip));
 
 	/* OPL2 init */
-	opl_reset();				/* Reset OPL chip */
+	opl_init();				/* Reset OPL chip */
 	opl_write(1, 0x20);
 	opl_write(8, 0);
 	opl_write(0xbd, regbd);
@@ -771,4 +766,3 @@ void lds_playsound(int inst_number, int channel_number, int tunehigh)
 	c->nextvol = c->glideto = c->finetune = c->vibcount = c->arp_pos = c->arp_count = 0;
 }
 
-// kate: tab-width 4; vim: set noet:
