@@ -1,5 +1,5 @@
 /* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -21,7 +21,12 @@
 
 #include "opentyr.h"
 
-extern char episode_name[6][31], difficulty_name[7][21], gameplay_name[5][26];
+#ifdef TYRIAN2000
+#define GAMEPLAY_NAME_COUNT 6
+#else
+#define GAMEPLAY_NAME_COUNT 5
+#endif
+extern char episode_name[6][31], difficulty_name[7][21], gameplay_name[GAMEPLAY_NAME_COUNT][26];
 
 bool select_gameplay( void );
 bool select_episode( void );
@@ -29,4 +34,3 @@ bool select_difficulty( void );
 
 #endif /* MENUS_H */
 
-// kate: tab-width 4; vim: set noet:
