@@ -1,5 +1,5 @@
 /*
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -23,9 +23,8 @@
 
 void mt_srand( unsigned long s );
 unsigned long mt_rand( void );
-double mt_rand_1(void);
-double mt_rand_lt1(void);
+float mt_rand_1( void );
+float mt_rand_lt1( void );
 
 #endif /* MTRAND_H */
 
-// kate: tab-width 4; vim: set noet:
