@@ -1,5 +1,5 @@
-/* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+/*
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -16,26 +16,29 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-#include "opentyr.h"
-#include "network.h"
-
 #include "episodes.h"
 #include "fonthand.h"
 #include "helptext.h"
 #include "joystick.h"
 #include "keyboard.h"
 #include "mainint.h"
-#include "newshape.h"
+#include "network.h"
 #include "nortvars.h"
+#include "opentyr.h"
 #include "picload.h"
+#include "sprite.h"
 #include "varz.h"
 #include "video.h"
 
-#include <SDL.h>
-#include <SDL_net.h>
-
 #include <assert.h>
 
+/*                              HERE BE DRAGONS!
+ *
+ * When I wrote this code I thought it was wonderful... that thought was very
+ * wrong.  It works, but good luck understanding how... I don't anymore.
+ *
+ * Hopefully it'll be rewritten some day.
+ */
 
 #define NET_VERSION       2            // increment whenever networking changes might create incompatability
 #define NET_PORT          1333         // UDP
@@ -56,31 +59,35 @@ char *network_opponent_host = NULL;
 Uint16 network_player_port = NET_PORT,
        network_opponent_port = NET_PORT;
 
-char *network_player_name = "",
-     *network_opponent_name = "";
+static char empty_string[] = "";
+char *network_player_name = empty_string,
+     *network_opponent_name = empty_string;
 
-UDPsocket socket;
-IPaddress ip;
+#ifdef WITH_NETWORK
+static UDPsocket socket;
+static IPaddress ip;
 
-UDPpacket *packet_out_temp, *packet_temp;
+UDPpacket *packet_out_temp;
+static UDPpacket *packet_temp;
 
 UDPpacket *packet_in[NET_PACKET_QUEUE] = { NULL },
           *packet_out[NET_PACKET_QUEUE] = { NULL };
 
-Uint16 last_out_sync = 0, queue_in_sync = 0, queue_out_sync = 0, last_ack_sync = 0;
-Uint32 last_in_tick = 0, last_out_tick = 0;
+static Uint16 last_out_sync = 0, queue_in_sync = 0, queue_out_sync = 0, last_ack_sync = 0;
+static Uint32 last_in_tick = 0, last_out_tick = 0;
 
-UDPpacket *packet_state_in[NET_PACKET_QUEUE] = { NULL },
-          *packet_state_in_xor[NET_PACKET_QUEUE] = { NULL },
-          *packet_state_out[NET_PACKET_QUEUE] = { NULL };
+UDPpacket *packet_state_in[NET_PACKET_QUEUE] = { NULL };
+static UDPpacket *packet_state_in_xor[NET_PACKET_QUEUE] = { NULL };
+UDPpacket *packet_state_out[NET_PACKET_QUEUE] = { NULL };
 
-Uint16 last_state_in_sync = 0, last_state_out_sync = 0;
-Uint32 last_state_in_tick = 0;
+static Uint16 last_state_in_sync = 0, last_state_out_sync = 0;
+static Uint32 last_state_in_tick = 0;
 
+static bool net_initialized = false;
 static bool connected = false, quit = false;
+#endif
 
-
-JE_integer thisPlayerNum = 0;  /* Player number on this PC (1 or 2) */
+uint thisPlayerNum = 0;  /* Player number on this PC (1 or 2) */
 
 JE_boolean haltGame = false;
 
@@ -90,6 +97,41 @@ JE_boolean moveOk;
 JE_boolean pauseRequest, skipLevelRequest, helpRequest, nortShipRequest;
 JE_boolean yourInGameMenuRequest, inGameMenuRequest;
 
+#ifdef WITH_NETWORK
+static void packet_copy( UDPpacket *dst, UDPpacket *src )
+{
+	void *temp = dst->data;
+	memcpy(dst, src, sizeof(*dst));
+	dst->data = temp;
+	memcpy(dst->data, src->data, src->len);
+}
+
+static void packets_shift_up( UDPpacket **packet, int max_packets )
+{
+		if (packet[0])
+		{
+			SDLNet_FreePacket(packet[0]);
+		}
+		for (int i = 0; i < max_packets - 1; i++)
+		{
+			packet[i] = packet[i + 1];
+		}
+		packet[max_packets - 1] = NULL;
+}
+
+static void packets_shift_down( UDPpacket **packet, int max_packets )
+{
+	if (packet[max_packets - 1])
+	{
+		SDLNet_FreePacket(packet[max_packets - 1]);
+	}
+	for (int i = max_packets - 1; i > 0; i--)
+	{
+		packet[i] = packet[i - 1];
+	}
+	packet[0] = NULL;
+}
+
 // prepare new packet for sending
 void network_prepare( Uint16 type )
 {
@@ -97,11 +139,25 @@ void network_prepare( Uint16 type )
 	SDLNet_Write16(last_out_sync, &packet_out_temp->data[2]);
 }
 
+// send packet but don't expect acknoledgment of delivery
+static bool network_send_no_ack( int len )
+{
+	packet_out_temp->len = len;
+
+	if (!SDLNet_UDP_Send(socket, 0, packet_out_temp))
+	{
+		printf("SDLNet_UDP_Send: %s\n", SDL_GetError());
+		return false;
+	}
+
+	return true;
+}
+
 // send packet and place it in queue to be acknowledged
 bool network_send( int len )
 {
 	bool temp = network_send_no_ack(len);
-	
+
 	Uint16 i = last_out_sync - queue_out_sync;
 	if (i < NET_PACKET_QUEUE)
 	{
@@ -112,32 +168,37 @@ bool network_send( int len )
 		fprintf(stderr, "warning: outbound packet queue overflow\n");
 		return false;
 	}
-	
+
 	last_out_sync++;
-	
+
 	if (network_is_sync())
 		last_out_tick = SDL_GetTicks();
-	
+
 	return temp;
 }
 
-// send packet but don't expect acknoledgment of delivery
-bool network_send_no_ack( int len )
+// send acknowledgement packet
+static int network_acknowledge( Uint16 sync )
 {
-	packet_out_temp->len = len;
-	
-	if (!SDLNet_UDP_Send(socket, 0, packet_out_temp))
-	{
-		printf("SDLNet_UDP_Send: %s\n", SDL_GetError());
-		return false;
-	}
-	
-	return true;
+	SDLNet_Write16(PACKET_ACKNOWLEDGE, &packet_out_temp->data[0]);
+	SDLNet_Write16(sync,               &packet_out_temp->data[2]);
+	network_send_no_ack(4);
+
+	return 0;
+}
+
+// activity lately?
+static bool network_is_alive( void )
+{
+	return (SDL_GetTicks() - last_in_tick < NET_TIME_OUT || SDL_GetTicks() - last_state_in_tick < NET_TIME_OUT);
 }
 
 // poll for new packets received, check that connection is alive, resend queued packets if necessary
 int network_check( void )
 {
+	if (!net_initialized)
+		return -1;
+
 	if (connected)
 	{
 		// timeout
@@ -146,18 +207,18 @@ int network_check( void )
 			if (!quit)
 				network_tyrian_halt(2, false);
 		}
-		
+
 		// keep-alive
 		static Uint32 keep_alive_tick = 0;
 		if (SDL_GetTicks() - keep_alive_tick > NET_KEEP_ALIVE)
 		{
 			network_prepare(PACKET_KEEP_ALIVE);
 			network_send_no_ack(4);
-			
+
 			keep_alive_tick = SDL_GetTicks();
 		}
 	}
-	
+
 	// retry
 	if (packet_out[0] && SDL_GetTicks() - last_out_tick > NET_RETRY)
 	{
@@ -166,10 +227,10 @@ int network_check( void )
 			printf("SDLNet_UDP_Send: %s\n", SDL_GetError());
 			return -1;
 		}
-		
+
 		last_out_tick = SDL_GetTicks();
 	}
-	
+
 	switch (SDLNet_UDP_Recv(socket, packet_temp))
 	{
 		case -1:
@@ -188,7 +249,7 @@ int network_check( void )
 						{
 							last_ack_sync = SDLNet_Read16(&packet_temp->data[2]);
 						}
-						
+
 						{
 							Uint16 i = SDLNet_Read16(&packet_temp->data[2]) - queue_out_sync;
 							if (i < NET_PACKET_QUEUE)
@@ -200,21 +261,21 @@ int network_check( void )
 								}
 							}
 						}
-						
+
 						// remove acknowledged packets from queue
 						while (packet_out[0] == NULL && (Uint16)(last_ack_sync - queue_out_sync) < NET_PACKET_QUEUE)
 						{
 							packets_shift_up(packet_out, NET_PACKET_QUEUE);
-							
+
 							queue_out_sync++;
 						}
-						
+
 						last_in_tick = SDL_GetTicks();
 						break;
-						
+
 					case PACKET_CONNECT:
 						queue_in_sync = SDLNet_Read16(&packet_temp->data[2]);
-						
+
 						for (int i = 0; i < NET_PACKET_QUEUE; i++)
 						{
 							if (packet_in[i])
@@ -223,7 +284,7 @@ int network_check( void )
 								packet_in[i] = NULL;
 							}
 						}
-						
+
 					case PACKET_DETAILS:
 					case PACKET_WAITING:
 					case PACKET_BUSY:
@@ -242,26 +303,26 @@ int network_check( void )
 								// under normal circumstances, this is okay
 							}
 						}
-						
+
 						network_acknowledge(SDLNet_Read16(&packet_temp->data[2]));
-						
+
 					case PACKET_KEEP_ALIVE:
 						last_in_tick = SDL_GetTicks();
 						break;
-						
+
 					case PACKET_QUIT:
 						if (!quit)
 						{
 							network_prepare(PACKET_QUIT);
 							network_send(4);  // PACKET_QUIT
 						}
-						
+
 						network_acknowledge(SDLNet_Read16(&packet_temp->data[2]));
-						
+
 						if (!quit)
 							network_tyrian_halt(1, true);
 						break;
-						
+
 					case PACKET_STATE:
 						// place packet in queue if within limits
 						{
@@ -274,7 +335,7 @@ int network_check( void )
 							}
 						}
 						break;
-						
+
 					case PACKET_STATE_XOR:
 						// place packet in queue if within limits
 						{
@@ -293,7 +354,7 @@ int network_check( void )
 							}
 						}
 						break;
-						
+
 					case PACKET_STATE_RESEND:
 						// resend requested state packet if still available
 						{
@@ -311,28 +372,18 @@ int network_check( void )
 							}
 						}
 						break;
-						
+
 					default:
 						fprintf(stderr, "warning: bad packet %d received\n", SDLNet_Read16(&packet_temp->data[0]));
 						return 0;
 						break;
 				}
-				
+
 				return 1;
 			}
 			break;
 	}
-	
-	return 0;
-}
 
-// send acknowledgement packet
-int network_acknowledge( Uint16 sync )
-{
-	SDLNet_Write16(PACKET_ACKNOWLEDGE, &packet_out_temp->data[0]);
-	SDLNet_Write16(sync,               &packet_out_temp->data[2]);
-	network_send_no_ack(4);
-	
 	return 0;
 }
 
@@ -342,12 +393,12 @@ bool network_update( void )
 	if (packet_in[0])
 	{
 		packets_shift_up(packet_in, NET_PACKET_QUEUE);
-		
+
 		queue_in_sync++;
-		
+
 		return true;
 	}
-	
+
 	return false;
 }
 
@@ -357,12 +408,6 @@ bool network_is_sync( void )
 	return (queue_out_sync - last_ack_sync == 1);
 }
 
-// activity lately?
-bool network_is_alive( void )
-{
-	return (SDL_GetTicks() - last_in_tick < NET_TIME_OUT || SDL_GetTicks() - last_state_in_tick < NET_TIME_OUT);
-}
-
 
 // prepare new state for sending
 void network_state_prepare( void )
@@ -374,7 +419,7 @@ void network_state_prepare( void )
 		packet_state_out[0] = SDLNet_AllocPacket(NET_PACKET_SIZE);
 		packet_state_out[0]->len = 28;
 	}
-	
+
 	SDLNet_Write16(PACKET_STATE, &packet_state_out[0]->data[0]);
 	SDLNet_Write16(last_state_out_sync, &packet_state_out[0]->data[2]);
 	memset(&packet_state_out[0]->data[4], 0, 28 - 4);
@@ -388,7 +433,7 @@ int network_state_send( void )
 		printf("SDLNet_UDP_Send: %s\n", SDL_GetError());
 		return -1;
 	}
-	
+
 	// send xor of last network_delay packets
 	if (network_delay > 1 && (last_state_out_sync + 1) % network_delay == 0 && packet_state_out[network_delay - 1] != NULL)
 	{
@@ -397,18 +442,18 @@ int network_state_send( void )
 		for (int i = 1; i < network_delay; i++)
 			for (int j = 4; j < packet_temp->len; j++)
 				packet_temp->data[j] ^= packet_state_out[i]->data[j];
-		
+
 		if (!SDLNet_UDP_Send(socket, 0, packet_temp))
 		{
 			printf("SDLNet_UDP_Send: %s\n", SDL_GetError());
 			return -1;
 		}
 	}
-	
+
 	packets_shift_down(packet_state_out, NET_PACKET_QUEUE);
-	
+
 	last_state_out_sync++;
-	
+
 	return 0;
 }
 
@@ -420,14 +465,14 @@ bool network_state_update( void )
 		return 0;
 	} else {
 		packets_shift_up(packet_state_in, NET_PACKET_QUEUE);
-		
+
 		packets_shift_up(packet_state_in_xor, NET_PACKET_QUEUE);
-		
+
 		last_state_in_sync++;
-		
+
 		// current xor packet index
 		int x = network_delay - (last_state_in_sync - 1) % network_delay - 1;
-		
+
 		// loop until needed packet is available
 		while (!packet_state_in[0])
 		{
@@ -454,21 +499,21 @@ bool network_state_update( void )
 					break;
 				}
 			}
-			
+
 			static Uint32 resend_tick = 0;
 			if (SDL_GetTicks() - last_state_in_tick > NET_RESEND && SDL_GetTicks() - resend_tick > NET_RESEND)
 			{
-				//SDLNet_Write16(PACKET_STATE_RESEND,    &packet_out_temp->data[0]);
-				//SDLNet_Write16(last_state_in_sync - 1, &packet_out_temp->data[2]);
+				SDLNet_Write16(PACKET_STATE_RESEND,    &packet_out_temp->data[0]);
+				SDLNet_Write16(last_state_in_sync - 1, &packet_out_temp->data[2]);
 				network_send_no_ack(4);  // PACKET_RESEND
-				
+
 				resend_tick = SDL_GetTicks();
 			}
-			
+
 			if (network_check() == 0)
 				SDL_Delay(1);
 		}
-		
+
 		if (network_delay > 1)
 		{
 			// process the current in packet against the xor queue
@@ -482,10 +527,10 @@ bool network_state_update( void )
 					packet_state_in_xor[x]->data[j] ^= packet_state_in[0]->data[j];
 			}
 		}
-		
+
 		last_state_in_tick = SDL_GetTicks();
 	}
-	
+
 	return 1;
 }
 
@@ -499,7 +544,7 @@ bool network_state_is_reset( void )
 void network_state_reset( void )
 {
 	last_state_in_sync = last_state_out_sync = 0;
-	
+
 	for (int i = 0; i < NET_PACKET_QUEUE; i++)
 	{
 		if (packet_state_in[i])
@@ -524,18 +569,19 @@ void network_state_reset( void )
 			packet_state_out[i] = NULL;
 		}
 	}
-	
+
 	last_state_in_tick = SDL_GetTicks();
 }
 
 
 // attempt to punch through firewall by firing off UDP packets at the opponent
 // exchange game information
-int network_connect( void ) {
+int network_connect( void )
+{
 	SDLNet_ResolveHost(&ip, network_opponent_host, network_opponent_port);
-	
+
 	SDLNet_UDP_Bind(socket, 0, &ip);
-	
+
 	Uint16 episodes = 0, episodes_local = 0;
 	assert(EPISODE_MAX <= 16);
 	for (int i = EPISODE_MAX - 1; i >= 0; i--)
@@ -544,11 +590,11 @@ int network_connect( void ) {
 		episodes |= (episodeAvail[i] != 0);
 	}
 	episodes_local = episodes;
-	
+
 	assert(NET_PACKET_SIZE - 12 >= 20 + 1);
 	if (strlen(network_player_name) > 20)
 		network_player_name[20] = '\0';
-	
+
 connect_reset:
 	network_prepare(PACKET_CONNECT);
 	SDLNet_Write16(NET_VERSION, &packet_out_temp->data[4]);
@@ -557,28 +603,28 @@ connect_reset:
 	SDLNet_Write16(thisPlayerNum,   &packet_out_temp->data[10]);
 	strcpy((char *)&packet_out_temp->data[12], network_player_name);
 	network_send(12 + strlen(network_player_name) + 1); // PACKET_CONNECT
-	
+
 	// until opponent sends connect packet
 	while (true)
 	{
 		push_joysticks_as_keyboard();
 		service_SDL_events(false);
-		
+
 		if (newkey && lastkey_sym == SDLK_ESCAPE)
 			network_tyrian_halt(0, false);
-		
+
 		// never timeout
 		last_in_tick = SDL_GetTicks();
-		
+
 		if (packet_in[0] && SDLNet_Read16(&packet_in[0]->data[0]) == PACKET_CONNECT)
 			break;
-		
+
 		network_update();
 		network_check();
-		
+
 		SDL_Delay(16);
 	}
-	
+
 connect_again:
 	if (SDLNet_Read16(&packet_in[0]->data[4]) != NET_VERSION)
 	{
@@ -595,36 +641,36 @@ connect_again:
 		fprintf(stderr, "error: player number conflicts with opponent's\n");
 		network_tyrian_halt(6, true);
 	}
-	
+
 	episodes = SDLNet_Read16(&packet_in[0]->data[8]);
 	for (int i = 0; i < EPISODE_MAX; i++) {
 		episodeAvail[i] &= (episodes & 1);
 		episodes >>= 1;
 	}
-	
+
 	network_opponent_name = malloc(packet_in[0]->len - 12 + 1);
 	strcpy(network_opponent_name, (char *)&packet_in[0]->data[12]);
-	
+
 	network_update();
-	
+
 	// until opponent has acknowledged
 	while (!network_is_sync())
 	{
 		service_SDL_events(false);
-		
+
 		// got a duplicate packet; process it again (but why?)
 		if (packet_in[0] && SDLNet_Read16(&packet_in[0]->data[0]) == PACKET_CONNECT)
 			goto connect_again;
-		
+
 		network_check();
-		
+
 		// maybe opponent didn't get our packet
 		if (SDL_GetTicks() - last_out_tick > NET_RETRY)
 			goto connect_reset;
-		
+
 		SDL_Delay(16);
 	}
-	
+
 	// send another packet since sometimes the network syncs without both connect packets exchanged
 	// there should be a better way to handle this
 	network_prepare(PACKET_CONNECT);
@@ -634,14 +680,14 @@ connect_again:
 	SDLNet_Write16(thisPlayerNum,   &packet_out_temp->data[10]);
 	strcpy((char *)&packet_out_temp->data[12], network_player_name);
 	network_send(12 + strlen(network_player_name) + 1); // PACKET_CONNECT
-	
+
 	connected = true;
-	
+
 	return 0;
 }
 
 // something has gone wrong :(
-void network_tyrian_halt( int err, bool attempt_sync )
+void network_tyrian_halt( unsigned int err, bool attempt_sync )
 {
 	const char *err_msg[] = {
 		"Quitting...",
@@ -652,115 +698,84 @@ void network_tyrian_halt( int err, bool attempt_sync )
 		"Network delay mismatch.",
 		"Network player number conflict.",
 	};
-	
+
 	quit = true;
-	
+
 	if (err >= COUNTOF(err_msg))
 		err = 0;
-	
+
 	fade_black(10);
-	
-	tempScreenSeg = VGAScreen = VGAScreenSeg;
-	
-	JE_loadPic(2, false);
-	JE_dString(JE_fontCenter(err_msg[err], SMALL_FONT_SHAPES), 140, err_msg[err], SMALL_FONT_SHAPES);
-	
+
+	VGAScreen = VGAScreenSeg;
+
+	JE_loadPic(VGAScreen, 2, false);
+	JE_dString(VGAScreen, JE_fontCenter(err_msg[err], SMALL_FONT_SHAPES), 140, err_msg[err], SMALL_FONT_SHAPES);
+
 	JE_showVGA();
 	fade_palette(colors, 10, 0, 255);
-	
+
 	if (attempt_sync)
 	{
 		while (!network_is_sync() && network_is_alive())
 		{
 			service_SDL_events(false);
-			
+
 			network_check();
 			SDL_Delay(16);
 		}
 	}
-	
+
 	if (err)
 	{
 		while (!JE_anyButton())
 			SDL_Delay(16);
 	}
-	
+
 	fade_black(10);
-	
+
 	SDLNet_Quit();
-	
+
 	JE_tyrianHalt(5);
 }
 
 int network_init( void )
 {
 	printf("Initializing network...\n");
-	
+
 	if (network_delay * 2 > NET_PACKET_QUEUE - 2)
 	{
 		fprintf(stderr, "error: network delay would overflow packet queue\n");
 		return -4;
 	}
-	
+
 	if (SDLNet_Init() == -1)
 	{
 		fprintf(stderr, "error: SDLNet_Init: %s\n", SDLNet_GetError());
 		return -1;
 	}
-	
+
 	socket = SDLNet_UDP_Open(network_player_port);
 	if (!socket)
 	{
 		fprintf(stderr, "error: SDLNet_UDP_Open: %s\n", SDLNet_GetError());
 		return -2;
 	}
-	
+
 	packet_temp = SDLNet_AllocPacket(NET_PACKET_SIZE);
 	packet_out_temp = SDLNet_AllocPacket(NET_PACKET_SIZE);
-	
+
 	if (!packet_temp || !packet_out_temp)
 	{
 		printf("SDLNet_AllocPacket: %s\n", SDLNet_GetError());
 		return -3;
 	}
-	
+
+	net_initialized = true;
+
 	return 0;
 }
 
-void packet_copy( UDPpacket *dst, UDPpacket *src )
-{
-	void *temp = dst->data;
-	memcpy(dst, src, sizeof(*dst));
-	dst->data = temp;
-	memcpy(dst->data, src->data, src->len);
-}
-
-void packets_shift_up( UDPpacket **packet, int max_packets )
-{
-		if (packet[0])
-		{
-			SDLNet_FreePacket(packet[0]);
-		}
-		for (int i = 0; i < max_packets - 1; i++)
-		{
-			packet[i] = packet[i + 1];
-		}
-		packet[max_packets - 1] = NULL;
-}
-
-void packets_shift_down( UDPpacket **packet, int max_packets )
-{
-	if (packet[max_packets - 1])
-	{
-		SDLNet_FreePacket(packet[max_packets - 1]);
-	}
-	for (int i = max_packets - 1; i > 0; i--)
-	{
-		packet[i] = packet[i - 1];
-	}
-	packet[0] = NULL;
-}
-
+#endif
 
 void JE_clearSpecialRequests( void )
 {
@@ -771,4 +786,3 @@ void JE_clearSpecialRequests( void )
 	nortShipRequest = false;
 }
 
-// kate: tab-width 4; vim: set noet:
