@@ -1,5 +1,5 @@
-/* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+/*
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -17,11 +17,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #include "config.h"
+#include "destruct.h"
 #include "editship.h"
 #include "episodes.h"
 #include "file.h"
-#include "fonthand.h"
+#include "font.h"
 #include "helptext.h"
+#include "hg_revision.h"
 #include "joystick.h"
 #include "jukebox.h"
 #include "keyboard.h"
@@ -30,14 +32,13 @@
 #include "mtrand.h"
 #include "musmast.h"
 #include "network.h"
-#include "newshape.h"
 #include "nortsong.h"
-#include "nortvars.h"
 #include "opentyr.h"
 #include "params.h"
 #include "picload.h"
 #include "scroller.h"
 #include "setup.h"
+#include "sprite.h"
 #include "tyrian2.h"
 #include "xmas.h"
 #include "varz.h"
@@ -52,20 +53,8 @@
 #include <stdlib.h>
 #include <time.h>
 
-
-const JE_byte shapereorderlist[7] = {1, 2, 5, 0, 3, 4, 6};
-
 const char *opentyrian_str = "OpenTyrian",
-           *opentyrian_version = "Classic revision " SVN_REV;
-const char *opentyrian_menu_items[] =
-{
-	"About OpenTyrian",
-	"Toggle Fullscreen",
-	"Scaler: None",
-	/* "Play Destruct", */
-	"Jukebox",
-	"Return to Main Menu"
-};
+           *opentyrian_version = HG_REV;
 
 /* zero-terminated strncpy */
 char *strnztcpy( char *to, const char *from, size_t count )
@@ -76,45 +65,76 @@ char *strnztcpy( char *to, const char *from, size_t count )
 
 void opentyrian_menu( void )
 {
-	int sel = 0;
-	const int maxSel = COUNTOF(opentyrian_menu_items) - 1;
-	bool quit = false, fade_in = true;
-	
-	int temp_scaler = scaler;
-	char buffer[100];
-	
-	fade_black(10);
-	JE_loadPic(13, false);
+	typedef enum
+	{
+		MENU_ABOUT = 0,
+		MENU_FULLSCREEN,
+		MENU_SCALER,
+		// MENU_DESTRUCT,
+		MENU_JUKEBOX,
+		MENU_RETURN,
+		MenuOptions_MAX
+	} MenuOptions;
 
-	JE_outTextAdjust(JE_fontCenter(opentyrian_str, FONT_SHAPES), 5, opentyrian_str, 15, -3, FONT_SHAPES, false);
+	static const char *menu_items[] =
+	{
+		"About OpenTyrian",
+		"Toggle Fullscreen",
+		"Scaler: None",
+		// "Play Destruct",
+		"Jukebox",
+		"Return to Main Menu",
+	};
+	bool menu_items_disabled[] =
+	{
+		false,
+		!can_init_any_scaler(false) || !can_init_any_scaler(true),
+		false,
+		// false,
+		false,
+		false,
+	};
+	
+	assert(COUNTOF(menu_items) == MenuOptions_MAX);
+	assert(COUNTOF(menu_items_disabled) == MenuOptions_MAX);
+
+	fade_black(10);
+	JE_loadPic(VGAScreen, 13, false);
+
+	draw_font_hv(VGAScreen, VGAScreen->w / 2, 5, opentyrian_str, large_font, centered, 15, -3);
 
 	memcpy(VGAScreen2->pixels, VGAScreen->pixels, VGAScreen2->pitch * VGAScreen2->h);
 
 	JE_showVGA();
 
 	play_song(36); // A Field for Mag
-	
+
+	MenuOptions sel = 0;
+
+	uint temp_scaler = scaler;
+
+	bool fade_in = true, quit = false;
 	do
 	{
 		memcpy(VGAScreen->pixels, VGAScreen2->pixels, VGAScreen->pitch * VGAScreen->h);
-		
-		for (int i = 0; i <= maxSel; i++)
+
+		for (MenuOptions i = 0; i < MenuOptions_MAX; i++)
 		{
-			const char *text = opentyrian_menu_items[i];
-			
-			if (i == 2) /* Scaler */
+			const char *text = menu_items[i];
+			char buffer[100];
+
+			if (i == MENU_SCALER)
 			{
 				snprintf(buffer, sizeof(buffer), "Scaler: %s", scalers[temp_scaler].name);
 				text = buffer;
 			}
-			
-			JE_outTextAdjust(JE_fontCenter(text, SMALL_FONT_SHAPES),
-			                 (i != maxSel) ? (i * 16 + 32) : 118, text,
-			                 15, (i != sel ? -4 : -2), SMALL_FONT_SHAPES, true);
+
+			int y = i != MENU_RETURN ? i * 16 + 32 : 118;
+			draw_font_hv(VGAScreen, VGAScreen->w / 2, y, text, normal_font, centered, 15, menu_items_disabled[i] ? -8 : i != sel ? -4 : -2);
 		}
-		
+
 		JE_showVGA();
-		
+
 		if (fade_in)
 		{
 			fade_in = false;
@@ -129,101 +149,125 @@ void opentyrian_menu( void )
 		{
 			switch (lastkey_sym)
 			{
-				case SDLK_UP:
-					sel--;
-					if (sel < 0)
-					{
-						sel = maxSel;
-					}
-					JE_playSampleNum(S_CURSOR);
-					break;
-				case SDLK_DOWN:
-					sel++;
-					if (sel > maxSel)
-					{
+			case SDLK_UP:
+				do
+				{
+					if (sel-- == 0)
+						sel = MenuOptions_MAX - 1;
+				}
+				while (menu_items_disabled[sel]);
+				
+				JE_playSampleNum(S_CURSOR);
+				break;
+			case SDLK_DOWN:
+				do
+				{
+					if (++sel >= MenuOptions_MAX)
 						sel = 0;
+				}
+				while (menu_items_disabled[sel]);
+				
+				JE_playSampleNum(S_CURSOR);
+				break;
+				
+			case SDLK_LEFT:
+				if (sel == MENU_SCALER)
+				{
+					do
+					{
+						if (temp_scaler == 0)
+							temp_scaler = scalers_count;
+						temp_scaler--;
 					}
+					while (!can_init_scaler(temp_scaler, fullscreen_enabled));
+					
 					JE_playSampleNum(S_CURSOR);
-					break;
-				case SDLK_LEFT:
-					if (sel == 2)
+				}
+				break;
+			case SDLK_RIGHT:
+				if (sel == MENU_SCALER)
+				{
+					do
 					{
-						do {
-							if (temp_scaler == 0)
-								temp_scaler = COUNTOF(scalers);
-							temp_scaler--;
-						} while ((display_surface->format->BitsPerPixel == 32 && scalers[temp_scaler].scaler32 == NULL) ||
-						         (display_surface->format->BitsPerPixel == 16 && scalers[temp_scaler].scaler16 == NULL));
-						if (display_surface->format->BitsPerPixel == 8)
+						temp_scaler++;
+						if (temp_scaler == scalers_count)
 							temp_scaler = 0;
-						else
-							JE_playSampleNum(S_CURSOR);
 					}
+					while (!can_init_scaler(temp_scaler, fullscreen_enabled));
+					
+					JE_playSampleNum(S_CURSOR);
+				}
+				break;
+				
+			case SDLK_RETURN:
+				switch (sel)
+				{
+				case MENU_ABOUT:
+					JE_playSampleNum(S_SELECT);
+
+					scroller_sine(about_text);
+
+					memcpy(VGAScreen->pixels, VGAScreen2->pixels, VGAScreen->pitch * VGAScreen->h);
+					JE_showVGA();
+					fade_in = true;
 					break;
-				case SDLK_RIGHT:
-					if (sel == 2)
+					
+				case MENU_FULLSCREEN:
+					JE_playSampleNum(S_SELECT);
+
+					if (!init_scaler(scaler, !fullscreen_enabled) && // try new fullscreen state
+						!init_any_scaler(!fullscreen_enabled) &&     // try any scaler in new fullscreen state
+						!init_scaler(scaler, fullscreen_enabled))    // revert on fail
 					{
-						do {
-							temp_scaler++;
-							if (temp_scaler == COUNTOF(scalers))
-								temp_scaler = 0;
-						} while ((display_surface->format->BitsPerPixel == 32 && scalers[temp_scaler].scaler32 == NULL) ||
-						         (display_surface->format->BitsPerPixel == 16 && scalers[temp_scaler].scaler16 == NULL));
-						if (display_surface->format->BitsPerPixel == 8)
-							temp_scaler = 0;
-						else
-							JE_playSampleNum(S_CURSOR);
+						exit(EXIT_FAILURE);
 					}
+					set_palette(colors, 0, 255); // for switching between 8 bpp scalers
 					break;
-				case SDLK_KP_ENTER:
-					switch (sel)
+					
+				case MENU_SCALER:
+					JE_playSampleNum(S_SELECT);
+
+					if (scaler != temp_scaler)
 					{
-						case 0: /* About */
-							JE_playSampleNum(S_SELECT);
-							
-							scroller_sine(about_text);
-							
-							memcpy(VGAScreen->pixels, VGAScreen2->pixels, VGAScreen->pitch * VGAScreen->h);
-							JE_showVGA();
-							fade_in = true;
-							break;
-						case 1: /* Fullscreen */
-							JE_playSampleNum(S_SELECT);
-							
-							fullscreen_enabled = !fullscreen_enabled;
-							reinit_video();
-							break;
-						case 2: /* Scaler */
-							JE_playSampleNum(S_SELECT);
-							
-							if (scaler != temp_scaler)
-							{
-								scaler = temp_scaler;
-								reinit_video();
-							}
-							break;
-						case 3: /* Jukebox */
-							JE_playSampleNum(S_SELECT);
-							
-							fade_black(10);
-							jukebox();
-							
-							memcpy(VGAScreen->pixels, VGAScreen2->pixels, VGAScreen->pitch * VGAScreen->h);
-							JE_showVGA();
-							fade_in = true;
-							break;
-						default: /* Return to main menu */
-							quit = true;
-							JE_playSampleNum(S_SPRING);
-							break;
+						if (!init_scaler(temp_scaler, fullscreen_enabled) &&   // try new scaler
+							!init_scaler(temp_scaler, !fullscreen_enabled) &&  // try other fullscreen state
+							!init_scaler(scaler, fullscreen_enabled))          // revert on fail
+						{
+							exit(EXIT_FAILURE);
+						}
+						set_palette(colors, 0, 255); // for switching between 8 bpp scalers
 					}
 					break;
-				case SDLK_ESCAPE:
+					
+				case MENU_JUKEBOX:
+					JE_playSampleNum(S_SELECT);
+
+					fade_black(10);
+					jukebox();
+
+					memcpy(VGAScreen->pixels, VGAScreen2->pixels, VGAScreen->pitch * VGAScreen->h);
+					JE_showVGA();
+					fade_in = true;
+					break;
+					
+				case MENU_RETURN:
 					quit = true;
 					JE_playSampleNum(S_SPRING);
 					break;
-				default:
+					
+				case MenuOptions_MAX:
+					assert(false);
 					break;
+				}
+				break;
+				
+			case SDLK_ESCAPE:
+				quit = true;
+				JE_playSampleNum(S_SPRING);
+				break;
+				
+			default:
+				break;
 			}
 		}
 	} while (!quit);
@@ -235,34 +279,22 @@ int main( int argc, char *argv[] )
 
 	printf("\nWelcome to... >> %s %s <<\n\n", opentyrian_str, opentyrian_version);
 
-	printf("Copyright (C) 2007-2009 The OpenTyrian Development Team\n\n");
+	printf("Copyright (C) 2007-2013 The OpenTyrian Development Team\n\n");
 
 	printf("This program comes with ABSOLUTELY NO WARRANTY.\n");
 	printf("This is free software, and you are welcome to redistribute it\n");
 	printf("under certain conditions.  See the file GPL.txt for details.\n\n");
-	
+
 	if (SDL_Init(0))
 	{
 		printf("Failed to initialize SDL: %s\n", SDL_GetError());
 		return -1;
 	}
 
-
-
-	system("/usr/bin/gconftool-2 -g /apps/osso/inputmethod/int_kb_layout > /home/user/int_kb_layout");
- 	FILE *archivo;
-	char caracteres[100];
-	int c;
- 	archivo = fopen("/home/user/int_kb_layout","r");
-	do { c = fscanf(archivo, "%s", caracteres); }
-    while (c != EOF); 
-	fclose(archivo);
-	if (caracteres!="us") system("/usr/bin/gconftool-2 -s /apps/osso/inputmethod/int_kb_layout us -t string");
-
-
-	
 	JE_loadConfiguration();
 
+	xmas = xmas_time();  // arg handler may override
+
 	JE_paramCheck(argc, argv);
 
 	JE_scanForEpisodes();
@@ -271,75 +303,93 @@ int main( int argc, char *argv[] )
 	init_keyboard();
 	init_joysticks();
 	printf("assuming mouse detected\n"); // SDL can't tell us if there isn't one
-	
-	xmas |= xmas_time();
+
 	if (xmas && (!dir_file_exists(data_dir(), "tyrianc.shp") || !dir_file_exists(data_dir(), "voicesc.snd")))
 	{
 		xmas = false;
-		
+
 		fprintf(stderr, "warning: Christmas is missing.\n");
 	}
-	
+
 	JE_loadPals();
 	JE_loadMainShapeTables(xmas ? "tyrianc.shp" : "tyrian.shp");
-	
-	tempScreenSeg = VGAScreen;
+
 	if (xmas && !xmas_prompt())
 	{
 		xmas = false;
-		
+
 		free_main_shape_tables();
 		JE_loadMainShapeTables("tyrian.shp");
 	}
-	
-	
+
+
 	/* Default Options */
 	youAreCheating = false;
 	smoothScroll = true;
 	loadDestruct = false;
-	
+
 	if (!audio_disabled)
 	{
 		printf("initializing SDL audio...\n");
-		
+
 		init_audio();
-		
+
 		load_music();
-		
+
 		JE_loadSndFile("tyrian.snd", xmas ? "voicesc.snd" : "voices.snd");
 	}
 	else
 	{
 		printf("audio disabled\n");
 	}
-	
+
 	if (record_demo)
 		printf("demo recording enabled (input limited to keyboard)\n");
 
-	megaData1 = malloc(sizeof(*megaData1));
-	megaData2 = malloc(sizeof(*megaData2));
-	megaData3 = malloc(sizeof(*megaData3));
-
 	JE_loadExtraShapes();  /*Editship*/
 
 	JE_loadHelpText();
 	/*debuginfo("Help text complete");*/
-	
+
 	if (isNetworkGame)
 	{
+#ifdef WITH_NETWORK
 		if (network_init())
 		{
 			network_tyrian_halt(3, false);
 		}
+#else
+		fprintf(stderr, "OpenTyrian was compiled without networking support.");
+		JE_tyrianHalt(5);
+#endif
 	}
-	
-	JE_main();
-	
-	// typically we don't get here, see JE_tyrianHalt()
-	deinit_video();
-	deinit_joysticks();
-	
+
+#ifdef NDEBUG
+	if (!isNetworkGame)
+		intro_logos();
+#endif
+
+	for (; ; )
+	{
+		JE_initPlayerData();
+		JE_sortHighScores();
+
+		if (JE_titleScreen(true))
+			break;  // user quit from title screen
+
+		if (loadDestruct)
+		{
+			JE_destructGame();
+			loadDestruct = false;
+		}
+		else
+		{
+			JE_main();
+		}
+	}
+
+	JE_tyrianHalt(0);
+
 	return 0;
 }
 
-// kate: tab-width 4; vim: set noet:
