@@ -1,5 +1,5 @@
-/* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+/*
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -21,8 +21,7 @@
 
 #include "opentyr.h"
 
-void JE_loadPic( JE_byte PCXnumber, JE_boolean storepal );
+void JE_loadPic(SDL_Surface *screen, JE_byte PCXnumber, JE_boolean storepal );
 
 #endif /* PICLOAD_H */
 
-// kate: tab-width 4; vim: set noet:
