@@ -1,5 +1,5 @@
 /* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -21,16 +21,6 @@
 
 #include "opentyr.h"
 
-
-#define starlib_MAX_STARS 1000
-#define MAX_TYPES 14
-
-struct JE_StarType
-{
-	JE_integer spX, spY, spZ;
-	JE_integer lastX, lastY;
-};
-
 void JE_starlib_main( void );
 void JE_wackyCol( void );
 void JE_starlib_init( void );
@@ -40,4 +30,3 @@ void JE_newStar( void );
 
 #endif /* STARLIB_H */
 
-// kate: tab-width 4; vim: set noet:
