@@ -1,5 +1,5 @@
-/* 
- * OpenTyrian Classic: A modern cross-platform port of Tyrian
+/*
+ * OpenTyrian: A modern cross-platform port of Tyrian
  * Copyright (C) 2007-2009  The OpenTyrian Development Team
  *
  * This program is free software; you can redistribute it and/or
@@ -24,25 +24,26 @@
 #include "SDL.h"
 
 #define vga_width 320
-#ifdef TARGET_GP2X
-#	define vga_height 200
-#else
-#	define vga_height 200
-#endif // TARGET_GP2X
+#define vga_height 200
 
 extern bool fullscreen_enabled;
 
-extern SDL_Surface *display_surface;
 extern SDL_Surface *VGAScreen, *VGAScreenSeg;
 extern SDL_Surface *game_screen;
 extern SDL_Surface *VGAScreen2;
 
 void init_video( void );
-void reinit_video( void );
+
+int can_init_scaler( unsigned int new_scaler, bool fullscreen );
+bool init_scaler( unsigned int new_scaler, bool fullscreen );
+bool can_init_any_scaler( bool fullscreen );
+bool init_any_scaler( bool fullscreen );
+
 void deinit_video( void );
-void JE_clr256( void );
+
+void JE_clr256( SDL_Surface * );
 void JE_showVGA( void );
+void scale_and_flip( SDL_Surface * );
 
 #endif /* VIDEO_H */
 
-// kate: tab-width 4; vim: set noet:
